from pydantic import ValidationError
from pydantic_core import PydanticCustomError


def discriminatorErrorWrapper(values, handler):
    """
    WrapValidator function that is applied on Union pydantic fields with discriminator
    to exclude discriminator value from exception location ('loc') on trace,
    and patch errors about discriminator field
    """
    try:
        return handler(values)
    except ValidationError as exc:
        errors = exc.errors()
        error = errors[0]
        if error["loc"]:
            error["loc"] = error["loc"][1:]
        error["type"] = PydanticCustomError(error["type"], error["msg"], error.get("ctx"))
        raise ValidationError.from_exception_data(exc.title, [error])
