import asyncio
from concurrent.futures import ThreadPoolExecutor

import onnxruntime as ort


def setFutureResult(future, result, error):
    """set result to asyncio future"""
    if not future.done():
        future.set_result((error, result))


def ortCallback(result, waiter: tuple[asyncio.AbstractEventLoop, asyncio.Future], error: str):
    """Callback for onnx thread"""
    loop = waiter[0]
    future = waiter[1]
    loop.call_soon_threadsafe(setFutureResult, future, result, error)


async def execute(output_names, input_feed, session: ort.InferenceSession, loop: asyncio.AbstractEventLoop):
    """
    Async execute onnx prediction. Real execution is provided in onnx thread.

    Args:
        output_names: name of the outputs
        input_feed: dictionary ``{ input_name: input_value }``
        session: ort session
        loop: current asyncio event loop
    Return:
        prediction result
    """
    future = loop.create_future()
    session.run_async(output_names=output_names, input_feed=input_feed, callback=ortCallback, user_data=(loop, future))
    error, res = await future
    # need error assert
    return res


class OnnxruntimeWrapper:
    def __init__(self, threadsNumber, onnxSession):
        self.loop = asyncio.get_running_loop()
        self.executor = ThreadPoolExecutor(max_workers=threadsNumber)
        self.onnxSession = onnxSession

    async def runAsync(self, func, *args):
        return await self.loop.run_in_executor(self.executor, func, *args)

    async def forward(self, input):
        return await execute(None, input, self.onnxSession, self.loop)
